@@ -4,6 +4,12 @@
 #include <time.h> /* for clock_gettime */
 #include <sys/time.h> /* for timeradd and friends */
 
+#if defined(__APPLE__) && defined(__MACH__)
+/* http://code.google.com/p/semicomplete/issues/detail?id=37
+ * OS X doesn't support clock_gettime (in at least OSX <= 10.6) */
+#  include "osx_hacks.h"
+#endif
+
 /* TODO(sissel): Refactor the madness.
  * The event look, delay and quiesce state handling, etc, are all pretty
  * intermingled. This needs a serious refactor into separate functions and
