@@ -1,11 +1,11 @@
 #include "xdo_cmd.h"
 
 /* Added 2007-07-28 - Lee Pumphret */
-int cmd_getwindowfocus(int argc, char **args) {
+int cmd_getwindowfocus(context_t *context) {
   int ret = 0;
   int get_toplevel_focus = 1;
-  Window wid = 0;
-  char *cmd = *args;
+  Window window = 0;
+  char *cmd = context->argv[0];
 
   int c;
   static struct option longopts[] = {
@@ -20,10 +20,12 @@ int cmd_getwindowfocus(int argc, char **args) {
     "         that has focus.\n";
   int option_index;
 
-  while ((c = getopt_long_only(argc, args, "fh", longopts, &option_index)) != -1) {
+  while ((c = getopt_long_only(context->argc, context->argv, "+fh",
+                               longopts, &option_index)) != -1) {
     switch (c) {
       case 'h':
         printf(usage, cmd);
+        consume_args(context, context->argc);
         return EXIT_SUCCESS;
         break;
       case 'f':
@@ -35,24 +37,27 @@ int cmd_getwindowfocus(int argc, char **args) {
     }
   }
 
-  argc -= optind;
-  args += optind;
+  consume_args(context, optind);
 
-  if (argc > 1) {
-    fprintf(stderr, usage, cmd);
-    return 1;
-  }
+  //if (context->argc > 0) {
+    //fprintf(stderr, usage, cmd);
+    //return 1;
+  //}
 
   if (get_toplevel_focus) {
-    ret = xdo_window_sane_get_focus(xdo, &wid);
+    ret = xdo_window_sane_get_focus(context->xdo, &window);
   } else {
-    ret = xdo_window_get_focus(xdo, &wid);
+    ret = xdo_window_get_focus(context->xdo, &window);
   }
 
   if (ret) {
     fprintf(stderr, "xdo_window_focus reported an error\n");
   } else { 
-    window_print(wid);
+    /* only print if we're the last command */
+    if (context->argc == 0) {
+      window_print(window);
+    }
+    window_save(context, window);
   }
 
   return ret;
