@@ -1,12 +1,19 @@
 #include "xdo_cmd.h"
 
+#define WINDOWMOVE_X_CURRENT (1 << 0)
+#define WINDOWMOVE_Y_CURRENT (1 << 1)
+#define WINDOWMOVE_RELATIVE (1 << 2)
+
 struct windowmove {
   Window window;
   int x;
   int y;
   int opsync;
+  int flags;
 };
 
+/* This function exists because at one time I had problems embedding certain
+ * blocks of code within macros (window_each). */
 static int _windowmove(context_t *context, struct windowmove *windowmove);
 
 int cmd_windowmove(context_t *context) {
@@ -18,19 +25,26 @@ int cmd_windowmove(context_t *context) {
   windowmove.y = 0;
   windowmove.opsync = 0;
   windowmove.window = CURRENTWINDOW;
+  windowmove.flags = 0;
 
   int c;
   typedef enum {
-    opt_unused, opt_help, opt_sync
+    opt_unused, opt_help, opt_sync, opt_relative
   } optlist_t;
   static struct option longopts[] = {
     { "help", no_argument, NULL, opt_help },
     { "sync", no_argument, NULL, opt_sync },
+    { "relative", no_argument, NULL, opt_relative },
     { 0, 0, 0, 0 },
   };
   static const char *usage = 
     "Usage: %s [options] [window=%1] x y\n"
-    "--sync    - only exit once the window has moved\n";
+    "--sync      - only exit once the window has moved\n"
+    "--relative  - make movements relative to the current window position"
+    "\n"
+    "If you use literal 'x' or 'y' for the x coordinates, then the current\n"
+    "coordinate will be used. This is useful for moving the window along\n"
+    "only one axis.\n";
 
   int option_index;
   while ((c = getopt_long_only(context->argc, context->argv, "+h",
@@ -45,6 +59,9 @@ int cmd_windowmove(context_t *context) {
       case opt_sync:
         windowmove.opsync = 1;
         break;
+      case opt_relative:
+        windowmove.flags |= WINDOWMOVE_RELATIVE;
+        break;
       default:
         fprintf(stderr, usage, cmd);
         return EXIT_FAILURE;
@@ -60,8 +77,17 @@ int cmd_windowmove(context_t *context) {
     return EXIT_FAILURE;
   }
 
-  windowmove.x = (int)strtol(context->argv[0], NULL, 0);
-  windowmove.y = (int)strtol(context->argv[1], NULL, 0);
+  if (context->argv[0][0] == 'x') {
+    windowmove.flags |= WINDOWMOVE_X_CURRENT;
+  } else {
+    windowmove.x = (int)strtol(context->argv[0], NULL, 0);
+  }
+
+  if (context->argv[1][0] == 'y') {
+    windowmove.flags |= WINDOWMOVE_Y_CURRENT;
+  } else {
+    windowmove.y = (int)strtol(context->argv[1], NULL, 0);
+  }
 
   consume_args(context, 2);  
 
@@ -77,19 +103,40 @@ static int _windowmove(context_t *context, struct windowmove *windowmove) {
   int orig_win_y = 0;
   int ret;
 
-  if (windowmove->opsync) {
+  /* Grab the current position of the window if we are moving synchronously
+   * or if we are moving along an axis. 
+   * That is, with --sync or x or y in args were literally 'x' or 'y'
+   * or if --relative is given*/
+  if (windowmove->opsync || windowmove->flags != 0) {
     xdo_get_window_location(context->xdo, windowmove->window,
                             &orig_win_x, &orig_win_y, NULL);
-    //fprintf(stderr, "%d,%d vs orig:%d,%d\n", windowmove->x, windowmove->y,
-            //orig_win_x, orig_win_y);
+    /* Break early if we don't need to move the window */
     if (orig_win_x == windowmove->x && orig_win_y == windowmove->y) {
-      /* Break early if we don't need to move the window */
       return 0;
     }
   }
 
-  ret = xdo_window_move(context->xdo, windowmove->window,
-                        windowmove->x, windowmove->y);
+  int target_x = windowmove->x;
+  int target_y = windowmove->y;
+
+
+  if (windowmove->flags & WINDOWMOVE_RELATIVE) {
+    target_x = orig_win_x + windowmove->x;
+    target_y = orig_win_y + windowmove->y;
+  }
+
+  if (windowmove->flags & WINDOWMOVE_X_CURRENT) {
+    target_x = orig_win_x;
+    xdotool_debug(context, "Using %d for x\n", windowmove->x);
+  }
+
+  if (windowmove->flags & WINDOWMOVE_Y_CURRENT) {
+    target_y = orig_win_y;
+    xdotool_debug(context, "Using %d for y\n", windowmove->y);
+  }
+
+
+  ret = xdo_window_move(context->xdo, windowmove->window, target_x, target_y);
   if (ret) {
     fprintf(stderr,
             "xdo_window_move reported an error while moving window %ld\n",
